import { defaultSyntax } from "../util/syntax.ts";
import { color, fg, fgBg } from "../util/colors.ts";
import { resolve, ref } from "../util/ouro.ts";

import { createTheme } from "../programs/emacs.ts";
import { pair, Conf } from "../programs/kitty.ts";


// This is a comment
export const palette = {
	gray: {
		0: color("#262726"),
		10: color("#EAEBEA"),
	},
	green: {
		3: color("#214521"),
		4: color("#285328"),
		6: color("#356E35"),
		8: color("#91CA91"),
		10: color("#C0E1C0"),
	},
	yellow: {
		8: color("#D4BF88"),
	},
	blue: {
		7: color("#9EBEE6"),
		10: color("#9EBEE6"),
	},
	cyan: {
		7: color("#7CDFC5"),
		10: color("#D0F4EA"),
	},
	orange: {
		7: color("#DFA77C"),
	},
	pink: {
		9: color("#E7B6C3"),
	},
	purple: {
		8: color("#D4B6E7"),
		10: color("#E4D0F0"),
	},
};
const p = palette;

export const emacsDark = createTheme(
	"everwild-dark",
	resolve({
		default: fgBg(p.gray[10], p.gray[0]),
		"mode-line": fgBg(p.gray[10], p.green[4]),
		region: ref("mode-line"),
		"vertico-current": ref("region"),
		"corfu-current": ref("region"),
	}),
	resolve({
		...defaultSyntax,
		comment: fg(p.cyan[7]),
		type: fg(p.blue[7]),
		function: fg(p.green[8]),
		constant: fg(p.purple[8]),
		string: fg(p.yellow[8]),
		variable: fg(p.orange[7]),
		keyword: fg(p.pink[9]),
	}),
);

export const kittyDark: Conf = resolve({
	background: p.gray[0],
	foreground: p.gray[10],
	selectionBackground: p.green[4],
	selectionForeground: ref("foreground"),
	cursor: ref("foreground"),
	cursorTextColor: "background",
	colors: {
		white: pair(p.gray[10], color("#FFFFFF")),
		black: pair(color("#000000"), p.gray[0]),
		blue: pair(p.blue[7], p.blue[10]),
		cyan: pair(p.cyan[7], p.cyan[10]),
		green: pair(p.green[8], p.green[10]),
		magenta: pair(p.purple[8], p.purple[10]),
		red: pair(p.pink[9], p.pink[9]),
		yellow: pair(p.yellow[8], p.yellow[8]),
	},
});
